﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using TddForWpfApplication.Services;
using TddForWpfApplication.Models;

namespace Tests
{
    [TestClass]
    public class LoginViewModelTest
    {
        private Mock<ILoginService> _mockedLoginService = new Mock<ILoginService>();
        private Mock<INavigationService> _mockedNavigationService = new Mock<INavigationService>();

        private readonly PropertyChangedHandler _propertyChangedHandler = new PropertyChangedHandler();

        [TestMethod]
        public void SettingLoginRaisesPropertyChanged()
        {
            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;

            model.Name = "Name";

            _propertyChangedHandler.Verify("Name");
        }

        [TestMethod]
        public void SettingPasswordRaisesPropertyChanged()
        {
            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;

            model.Password = "Password";

            _propertyChangedHandler.Verify("Password");
        }

        [TestMethod]
        public void SettingErrorMessageRaisesPropertyChanged()
        {
            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;

            model.ErrorMessage = "Error";

            _propertyChangedHandler.Verify("ErrorMessage");
        }

        [TestMethod]
        public void LoginWithCorrectCredentialsOpensMainView()
        {
            _mockedLoginService.Setup(s => s.Login("User", "Password"))
                               .Returns(LoginResult.Success);

            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.Name = "User";
            model.Password = "Password";

            model.LoginCommand.Execute(null);

            _mockedLoginService.Verify(s => s.Login("User", "Password"), Times.Once);
            _mockedNavigationService.Verify(s => s.NavigateTo(NavigationRegion.StartPage), Times.Once);
        }

        [TestMethod]
        public void LoginWithWrongCredentialsShowsError()
        {
            _mockedLoginService.Setup(s => s.Login("User", "WrongPassword"))
                               .Returns(LoginResult.Error("Du kommst hier nicht rein"));

            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;
            model.Name = "User";
            model.Password = "WrongPassword";

            model.LoginCommand.Execute(null);

            Assert.AreEqual("Du kommst hier nicht rein", model.ErrorMessage);
            _mockedLoginService.Verify(s => s.Login("User", "WrongPassword"), Times.Once);
            _mockedNavigationService.Verify(s => s.NavigateTo(NavigationRegion.StartPage), Times.Never);
            _propertyChangedHandler.Verify("ErrorMessage");
        }

        [TestMethod]
        public void SetttingNewNameClearsErrorMessage()
        {
            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;
            model.ErrorMessage = "Error";

            model.Name = "User";

            Assert.AreEqual(string.Empty, model.ErrorMessage);
            _propertyChangedHandler.Verify("ErrorMessage");
        }

        [TestMethod]
        public void SetttingNewPasswordClearsErrorMessage()
        {
            var model = new LoginViewModel(_mockedLoginService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;
            model.ErrorMessage = "Error";

            model.Password = "WrongPassword";

            Assert.AreEqual(string.Empty, model.ErrorMessage);
            _propertyChangedHandler.Verify("ErrorMessage");
        }
    }
}
