﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TddForWpfApplication.Models;
using TddForWpfApplication.Services;

namespace Tests
{
    [TestClass]
    public class MainViewModelTest
    {
        private readonly Mock<IDataService> _mockedDataService = new Mock<IDataService>();
        private readonly Mock<INavigationService> _mockedNavigationService = new Mock<INavigationService>();

        private readonly PropertyChangedHandler _propertyChangedHandler = new PropertyChangedHandler();

        [TestMethod]
        public void ItemsAreInitializedInConstructor()
        {
            SetupDataListOneItem();

            var model = new MainViewModel(_mockedDataService.Object, _mockedNavigationService.Object);

            Assert.AreEqual(1, model.Items.Count);
            Assert.AreEqual("Test", model.Items.First().Name);
        }

        [TestMethod]
        public void SelectItemPutsSelectedItemIntoProperty()
        {
            SetupDataListOneItem();
            var model = new MainViewModel(_mockedDataService.Object, _mockedNavigationService.Object);
            model.PropertyChanged += _propertyChangedHandler.PropertyChanged;

            model.SelectItemCommand.Execute(model.Items.First());

            Assert.AreEqual("Test", model.SelectedItem.Name);
            Assert.AreEqual(new DateTime(2013, 1, 1), model.SelectedItem.Date);
            _propertyChangedHandler.Verify("SelectedItem");
        }

        private void SetupDataListOneItem()
        {
            _mockedDataService.Setup(s => s.GetDataList())
                              .Returns(new List<Data>
                                       {
                                           new Data { Name = "Test", Date = new DateTime(2013, 1, 1) }
                                       });
        }
    }
}
